/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2007        */
/*-----------------------------------------------------------------------*/
/* This is a stub disk I/O module that acts as front end of the existing */
/* disk I/O modules and attach it to FatFs module with common interface. */
/*-----------------------------------------------------------------------*/
#include "stm32f4xx.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "diskio.h"


//#define SDIO_NATIVE


#define	BUFFERING			1		// buffering cause alignment ...

#define BLOCK_SIZE          512		// Block Size in Bytes





#ifdef	SDIO_NATIVE
#include "sdio.h"
#else
#include "stm32f4_discovery_sdio_sd.h"
#endif




#if	BUFFERING
	static BYTE  __attribute__ ((aligned)) 	diskbuf[BLOCK_SIZE *2];
#endif






/*-----------------------------------------------------------------------*/
/* Inidialize a Drive                                                    */

DSTATUS disk_initialize (
        BYTE drv                                /* Physical drive nmuber (0..) */
)
{
        /* Supports only single drive */
        if (drv)
        {
                return STA_NOINIT;
        }
/*-------------------------- SD Init ----------------------------- */
#ifdef	SDIO_NATIVE
		SD_LowLevel_Init();		//Initialize PINS, vector table and SDIO interface
		SD_Init();				//After return from this function sd is in transfer mode.
		return RES_OK;
#else
        SD_Error  Status;

		Status = SD_Init();
		if (Status!=SD_OK )
		{
				return STA_NOINIT;

		}
		else
		{

				return RES_OK;
		}
#endif
}



/*-----------------------------------------------------------------------*/
/* Return Disk Status                                                    */

DSTATUS disk_status (
        BYTE drv                /* Physical drive nmuber (0..) */
)
{
        return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */

DRESULT disk_read (
        BYTE drv,               /* Physical drive nmuber (0..) */
        BYTE *buff,             /* Data buffer to store read data */
        DWORD sector,   		/* Sector address (LBA) */
        BYTE count              /* Number of sectors to read (1..255) */
)
{
	SD_Error rc;
#if	BUFFERING
	BYTE	*buf = diskbuf;
	BYTE	*bufalt = NULL;

	if((int)buff & 0x3  || ((int)buff & 0xf0000000) == 0x10000000)
	{
		// not aligned buffer
		//printf("::diskio: reading not aligned buffer.blocks: %d\n", count);

		if(count)
		{
			while(count--)
			{
				if((rc = SD_ReadBlock(buf, sector*BLOCK_SIZE, BLOCK_SIZE)) != SD_OK)
				{
					return RES_ERROR;
				}
				sector++;

				if(bufalt != NULL)
				{
					memcpy(buff, bufalt, BLOCK_SIZE);
					buff += BLOCK_SIZE;
				}
				bufalt = buf;

				// fill next buffer
				if(buf == diskbuf)		buf = diskbuf +BLOCK_SIZE;
				else					buf = diskbuf;

				while(SD_GetTransferState() == SD_TRANSFER_BUSY);
			}
			// fill last block
			memcpy(buff, bufalt, BLOCK_SIZE);
		}
	}
	else
#endif
	{
		// aligned buffer
		//printf("::diskio: aligned buffer.\n");
		while(count--)
		{
			if((rc = SD_ReadBlock(buff, sector*BLOCK_SIZE, BLOCK_SIZE)) != SD_OK)
			{
				return RES_ERROR;
			}
			buff += BLOCK_SIZE;
			sector++;

			while(SD_GetTransferState() == SD_TRANSFER_BUSY);
		}
	}
	return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */

#if _READONLY == 0
DRESULT disk_write (
        BYTE drv,                       /* Physical drive nmuber (0..) */
        const BYTE *buff,       /* Data to be written */
        DWORD sector,           /* Sector address (LBA) */
        BYTE count                      /* Number of sectors to write (1..255) */
)
{
	SD_Error rc;
#if	BUFFERING
	BYTE	*buf = diskbuf;

	if((int)buff & 0x3  || ((int)buff & 0xf0000000) == 0x10000000)
	{
		// not aligned buffer
		//printf("::diskio: writing not aligned buffer.blocks: %d\n", count);

		memcpy(buf, buff, BLOCK_SIZE);
		while(count--)
		{
			if((rc = SD_WriteBlock((uint8_t *)buf,sector*BLOCK_SIZE, BLOCK_SIZE)) != SD_OK)
			{
				return RES_ERROR;
			}
			sector++;
			buff += BLOCK_SIZE;

			// fill next buffer
			if(count > 0)
			{
				if(buf == diskbuf)		buf = diskbuf +BLOCK_SIZE;
				else					buf = diskbuf;
				memcpy(buf, buff, BLOCK_SIZE);
			}

			while(SD_GetTransferState() == SD_TRANSFER_BUSY);
		}
	}
	else
#endif
	{
		// aligned buffer
		//printf("::diskio: aligned buffer.\n");
		while(count--)
		{
			if((rc = SD_WriteBlock((uint8_t *)buff,sector*BLOCK_SIZE, BLOCK_SIZE)) != SD_OK)
			{
				return RES_ERROR;
			}
			buff += BLOCK_SIZE;
			sector++;

			while(SD_GetTransferState() == SD_TRANSFER_BUSY);
		}
	}
	return RES_OK;
}
#endif /* _READONLY */




/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */

DRESULT disk_ioctl (
        BYTE drv,               /* Physical drive nmuber (0..) */
        BYTE ctrl,              /* Control code */
        void *buff              /* Buffer to send/receive control data */
)
{
        return RES_OK;
}

/*-----------------------------------------------------------------------*/
/* Get current time                                                      */
/*-----------------------------------------------------------------------*/
DWORD get_fattime(void){
        return 0;
}
