/**********************************************************************************/
/*    Basecode for Discovery 32F4 board											  */
/*	  Board: STM32F407 Discovery				             					  */
/*    Manufacture: ST		                                                   	  */
/*	  COPYRIGHT (C) 2012		Ing. Thomas Winkler	<t.winkler@aon.at>			  */
/*	  Copyright (C) 2007-2012	Ingo Korb 			<ingo@akana.de>				  */
/*    Built with CooCox CoIDE v1.5.0  					                          */
/**********************************************************************************/
/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License only.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


   f_io.h: Definitions for the FAT operations
*/

#include "ff.h"			/* FatFs configurations and declarations */
#include "diskio.h"		/* Declarations of low level disk I/O functions */
#include "f_io.h"			/* FatFs configurations and declarations */



#if _USE_STRFUNC >= 1


#if !_FS_READONLY
#include <stdarg.h>
/*-----------------------------------------------------------------------*/
/* Put a character to the file                                           */
/*-----------------------------------------------------------------------*/
int fputc (
    int chr,    /* A character to be output */
    FIL* fil    /* Ponter to the file object */
)
{
    UINT bw;
    char c;


#if _USE_STRFUNC >= 2
    if (chr == '\n') fputc ('\r', fil); /* LF -> CRLF conversion */
#endif
    if (!fil) { /* Special value may be used to switch the destination to any other device */
    /*  put_console(chr);   */
        return chr;
    }
    c = (char)chr;
    f_write(fil, &c, 1, &bw);   /* Write a byte to the file */
    return bw ? chr : EOF;      /* Return the resulut */
}




/*-----------------------------------------------------------------------*/
/* Put a formatted string to the file                                    */
/*-----------------------------------------------------------------------*/
int fprintf (
    FIL* fil,           /* Pointer to the file object */
    const char* str,    /* Pointer to the format string */
    ...                 /* Optional arguments... */
)
{
    va_list arp;
    UCHAR c, f, r;
    ULONG val;
    char s[16];
    int i, w, res, cc;


    va_start(arp, str);

    for (cc = res = 0; cc != EOF; res += cc) {
        c = *str++;
        if (c == 0) break;          /* End of string */
        if (c != '%') {             /* Non escape cahracter */
            cc = fputc(c, fil);
            if (cc != EOF) cc = 1;
            continue;
        }
        w = f = 0;
        c = *str++;
        if (c == '0') {             /* Flag: '0' padding */
            f = 1; c = *str++;
        }
        while (c >= '0' && c <= '9') {  /* Precision */
            w = w * 10 + (c - '0');
            c = *str++;
        }
        if (c == 'l') {             /* Prefix: Size is long int */
            f |= 2; c = *str++;
        }
        if (c == 's') {             /* Type is string */
            cc = fputs(va_arg(arp, char*), fil);
            continue;
        }
        if (c == 'c') {             /* Type is character */
            cc = fputc(va_arg(arp, int), fil);
            if (cc != EOF) cc = 1;
            continue;
        }
        r = 0;
        if (c == 'd') r = 10;       /* Type is signed decimal */
        if (c == 'u') r = 10;       /* Type is unsigned decimal */
        if (c == 'X') r = 16;       /* Type is unsigned hexdecimal */
        if (r == 0) break;          /* Unknown type */
        if (f & 2) {                /* Get the value */
            val = (ULONG)va_arg(arp, long);
        } else {
            val = (c == 'd') ? (ULONG)(long)va_arg(arp, int) : (ULONG)va_arg(arp, unsigned int);
        }
        /* Put numeral string */
        if (c == 'd') {
            if (val >= 0x80000000) {
                val = 0 - val;
                f |= 4;
            }
        }
        i = sizeof(s) - 1; s[i] = 0;
        do {
            c = (UCHAR)(val % r + '0');
            if (c > '9') c += 7;
            s[--i] = c;
            val /= r;
        } while (i && val);
        if (i && (f & 4)) s[--i] = '-';
        w = sizeof(s) - 1 - w;
        while (i && i > w) s[--i] = (f & 1) ? '0' : ' ';
        cc = fputs(&s[i], fil);
    }

    va_end(arp);
    return (cc == EOF) ? cc : res;
}




#endif /* !_FS_READONLY */
#endif /* _USE_STRFUNC >= 1*/
