/**********************************************************************************/
/*    Basecode for Discovery 32F4 board											  */
/*	  Board: STM32F407 Discovery				             					  */
/*    Manufacture: ST		                                                   	  */
/*	  COPYRIGHT (C) 2012														  */
/*    Designed by: Ing. Thomas Winkler			                                  */
/*    Built with CooCox CoIDE v1.5.0  					                          */
/**********************************************************************************/
/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License only.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   uart.h: uart/usart managment
*/
#ifndef USART_H_
#define USART_H_







// CONST
#define 	COMBUF_SIZE_TX		256
#define 	COMBUF_SIZE_RX		256

#define 	BAUD 	115200
//#define 	BAUD 	57600
//#define 	BAUD 	38400
//#define 	BAUD 	19200
//#define 	BAUD 	9600





#define RingbBytesInBuf(rb)		((rb)->cnt)
#define RingbBytesFree(rb)		((rb)->size - (rb)->cnt)



#define	_kbhit()		uart_rx()
#define	_getch()		uart_getch()
#define	gets(b)			uart_gets(b,sizeof(b))

#ifdef putchar
#undef putchar
#define putchar(ch)		uart_putchar(ch)
#endif





/* USER_TIMEOUT value for waiting loops. This timeout is just guarantee that the
   application will not remain stuck if the USART communication is corrupted.
   You may modify this timeout value depending on CPU frequency and application
   conditions (interrupts routines, number of data to transfer, baudrate, frequence
   CPU...). */
#define USER_TIMEOUT                    ((uint32_t)0x64) /* Waiting 1s */

/* USART Mode Selected */
#define USART_MODE_TRANSMITTER           0x00
#define USART_MODE_RECEIVER              0x01

/* USART Transaction Type */
#define USART_TRANSACTIONTYPE_CMD        0x00
#define USART_TRANSACTIONTYPE_DATA       0x01


/* Definition for USARTx resources ********************************************/
#define USART 	2

#if USART == 2
#define USARTx                           USART2
#define USARTx_CLK                       RCC_APB1Periph_USART2
#define USARTx_CLK_INIT                  RCC_APB1PeriphClockCmd
#define USARTx_IRQn                      USART2_IRQn
#define USARTx_IRQHandler                USART2_IRQHandler

#define USARTx_TX_PIN                    GPIO_Pin_2
#define USARTx_TX_GPIO_PORT              GPIOA
#define USARTx_TX_GPIO_CLK               RCC_AHB1Periph_GPIOA
#define USARTx_TX_SOURCE                 GPIO_PinSource2
#define USARTx_TX_AF                     GPIO_AF_USART3

#define USARTx_RX_PIN                    GPIO_Pin_3
#define USARTx_RX_GPIO_PORT              GPIOA
#define USARTx_RX_GPIO_CLK               RCC_AHB1Periph_GPIOA
#define USARTx_RX_SOURCE                 GPIO_PinSource3
#define USARTx_RX_AF                     GPIO_AF_USART2
#endif

#if USART == 3
#define USARTx                           USART3
#define USARTx_CLK                       RCC_APB1Periph_USART3
#define USARTx_CLK_INIT                  RCC_APB1PeriphClockCmd
#define USARTx_IRQn                      USART3_IRQn
#define USARTx_IRQHandler                USART3_IRQHandler

#define USARTx_TX_PIN                    GPIO_Pin_10
#define USARTx_TX_GPIO_PORT              GPIOC
#define USARTx_TX_GPIO_CLK               RCC_AHB1Periph_GPIOC
#define USARTx_TX_SOURCE                 GPIO_PinSource10
#define USARTx_TX_AF                     GPIO_AF_USART3

#define USARTx_RX_PIN                    GPIO_Pin_11
#define USARTx_RX_GPIO_PORT              GPIOC
#define USARTx_RX_GPIO_CLK               RCC_AHB1Periph_GPIOC
#define USARTx_RX_SOURCE                 GPIO_PinSource11
#define USARTx_RX_AF                     GPIO_AF_USART3
#endif





// STRUCTS
typedef struct
{
	volatile uint16_t	wp;				// write idx
	volatile uint16_t	rp;				// read idx
	volatile uint16_t	cnt;			// bytes in buffer
	volatile uint16_t	size;			// buffer size
	uint8_t	buf[0];
} st_ringbuf;




void uart_init(void);
//void uart_enable(void);



void RingbInit(st_ringbuf *rb, uint16_t);
int8_t RingbWrite(st_ringbuf *rb, char d);
int8_t RingbWriteBuf(st_ringbuf *rb, uint8_t *buf, uint16_t anz);
int16_t RingbRead(st_ringbuf *rb);
int8_t RingbReadBuf(st_ringbuf *rb, uint8_t *buf, uint16_t anz);


void uart_send(const char *text, int len);
void uart_putc(char ch);
void uart_putchar(char ch);
void uart_puts(const char *text);
void uart_putcrlf(void);

void uart_receive(char *buf, int cnt);
char *uart_gets(char *buf, int size);
int uart_getch(void);
int uart_rx(void);

#endif


