/**********************************************************************************/
/*    Sample code for Discovery 32F4 board										  */
/*	  Board: STM32F407 Discovery				             					  */
/*    Manufacture: ST		                                                   	  */
/*	  COPYRIGHT (C) 2012														  */
/*    Designed by: Ing. Thomas Winkler			                                  */
/*    Built with CooCox CoIDE v1.5.0  					                          */
/**********************************************************************************/
#include "discovery.h"
#include "gpio.h"
#include "sTimer.h"
//#include "iec.h"




#define	_DELAY		400




void BlinkExpIni()
{
	InitDiscovery();
}



//
// blink all LED's
//
int BlinkExpLoop1()
{
	static uint8_t	state = 0;
	//static uint8_t	cnt = 0;
	static STIMER	t1;

	switch(state)
	{
	case 0:
		/* Output high level */
		SetLED(ALL_LEDS, 1);
		SET_TIMER_MS(t1, _DELAY);
		state++;
		break;
	case 1:
		if(!CHK_TIMER_MS(t1))
		{
			/* Output low level */
			SetLED(ALL_LEDS, 0);
			SET_TIMER_MS(t1, _DELAY);
			state++;
		}
		break;
	case 2:
		if(!CHK_TIMER_MS(t1))
		{
			state = 0;
			return 1;
		}
		break;
	}
	return 0;
}


//
// Set all LED's, shift left, right
//
int BlinkExpLoop2()
{
	static uint16_t	bits;
	static uint8_t	state = 0;
	static uint8_t	cnt = 0;
	static STIMER	t1;

	switch(state)
	{
	case 1:
		SetLED(bits, 2);
		SET_TIMER_MS(t1, _DELAY);
		state++;
		break;
	case 2:
		if(CHK_TIMER_MS(t1))
			break;

		if(cnt++ < LED_CNT +1)
		{
			bits <<= 1;
			state--;
			break;
		}
		bits = 0;
		cnt = 0;
		state++;
	case 3:
		bits |= LED_BLUE;
		SetLED(bits, 2);
		SET_TIMER_MS(t1, _DELAY);
		state++;
		break;
	case 4:
		if(CHK_TIMER_MS(t1))
			break;

		if(cnt++ < LED_CNT -1)
		{
			bits >>= 1;
			state--;
			break;
		}
		bits = ALL_LEDS;
		cnt = 0;
		state++;
	case 5:
		SetLED(bits, 2);
		SET_TIMER_MS(t1, _DELAY);
		state++;
		break;
	case 6:
		if(CHK_TIMER_MS(t1))
			break;

		if(cnt++ < LED_CNT +1)
		{
			bits >>= 1;
			state--;
			break;
		}
		bits = 0;
		cnt = 0;
		state++;
	case 7:
		bits |= LED_GREEN;
		SetLED(bits, 2);
		SET_TIMER_MS(t1, _DELAY);
		state++;
		break;
	case 8:
		if(CHK_TIMER_MS(t1))
			break;

		if(cnt++ < LED_CNT )
		{
			bits <<= 1;
			state--;
			break;
		}
		state = 0;
		return 1;
	default:
		bits = ALL_LEDS;
		state++;
	}
	return 0;
}


//
// move all LEDs to left or right
//
int BlinkExpLoop3()
{
	static uint16_t	bits;
	static uint8_t	state = 0;
	static uint8_t	cnt = 0;
	static STIMER	t1;

	switch(state)
	{
	case 0:
		bits = LED_GREEN;
		cnt = 0;
		state++;
	case 1:
		SetLED(bits, 2);
		SET_TIMER_MS(t1, _DELAY);
		state++;
		break;
	case 2:
		if(CHK_TIMER_MS(t1))
			break;

		if(cnt++ < LED_CNT )
		{
			bits <<= 1;
			state--;
			break;
		}
		state = 0;
		return 1;
	}
	return 0;
}




//
// set drive led
//
void GpioDriveLED(uint8_t stat)
{
	SetLED(LED_GREEN, stat?1:0);
}



//
// set drive motor
//
void GpioMotorLED(uint8_t stat)
{
	SetLED(LED_ORANGE, stat?1:0);
}



//
// set write mode
//
void GpioWriteLED(uint8_t stat)
{
	SetLED(LED_RED, stat?1:0);
}



//
// for test purposes
//
void GpioDebugLED(uint8_t stat)
{
	SetLED(LED_BLUE, stat?1:0);
}



//
// iec write
//
void GpioIecOut(uint8_t stat)
{
	uint16_t bitValue;

	bitValue =	((stat & ATN_OUT)		? 0 : IEC_ATNo) |
				((stat & CLK_OUT_VIA)	? 0 : IEC_CLKo) |
				((stat & DATA_OUT_VIA)	? 0 : IEC_DATAo);
	GPIO_Write(IEC_OUT, bitValue);
}



//
// read iec lines
//
uint8_t GpioIecIn()
{
	uint16_t bitValue = GPIO_ReadInputData(IEC_INP);

	return		((bitValue & IEC_ATNi)	? ATN_IN_VIA : 0) |
				((bitValue & IEC_CLKi)	? CLK_IN_VIA : 0) |
				((bitValue & IEC_DATAi) ? DATA_IN_VIA : 0);
}




//
// PIN change interrupts
//
//extern volatile uint8_t	iecStat;
//extern volatile uint8_t	iecInvalidate;
void IecPinChangeAtn(void);
void IecPinChange(void);
void EXTI0_IRQHandler(void)
{
	if(EXTI->PR & EXTI_Line0)
	{
		// clear pending bit
		EXTI->PR = EXTI_Line0;

		/*if((GPIO_ReadInputData(IEC_INP) & IEC_ATNi))
			iecStat |= ATN_IN_VIA;
		else
			iecStat &= ~ATN_IN_VIA;*/
		//iecInvalidate = 1;
		IecPinChangeAtn();
	}
}
void EXTI1_IRQHandler(void)
{
	if(EXTI->PR & EXTI_Line1)
	{
		// clear pending bit
		EXTI->PR = EXTI_Line1;

		/*if((GPIO_ReadInputData(IEC_INP) & IEC_DATAi))
			iecStat |= DATA_IN_VIA;
		else
			iecStat &= ~DATA_IN_VIA;*/
		//iecInvalidate = 1;
		IecPinChange();
	}
}
void EXTI2_IRQHandler(void)
{
	if(EXTI->PR & EXTI_Line2)
	{
		// clear pending bit
		EXTI->PR = EXTI_Line2;

		/*if((GPIO_ReadInputData(IEC_INP) & IEC_CLKi))
			iecStat |= CLK_IN_VIA;
		else
			iecStat &= ~CLK_IN_VIA;*/
		IecPinChange();
	}
}
