/********************************************************************************/
/*	Basecode for 1541-Emul (floppy drive emulator)								*/
/*	Board: STM32F407 Discovery													*/
/*	COPYRIGHT (C) 2012		Ing. Thomas Winkler	<t.winkler@aon.at>				*/
/*  Built with CooCox CoIDE v1.5.0  											*/
/********************************************************************************/
/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License only.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   sTimer.c: soft timer, systick, delay management
*/

#include "stm32f4xx_conf.h"
#include "discovery.h"
#include "sTimer.h"



static volatile uint32_t MillisCounter = 0;


void ResetStimer(void)
{
	//cnt = 0;
	MillisCounter = 0;
	//MicrosCounter = 0;
}



/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
//void TimingDelay10us_Decrement(void)
void SysTick_Handler(void)
{
	// every 1ms
	MillisCounter++;
}

void Delay_us(__IO uint32_t nTime)
{
	STIMER	timer;
	SET_TIMER_US(timer, nTime);
	while(CHK_TIMER_US(timer));
}

void Delay_Ms(__IO uint32_t nTime)
{ 
	STIMER	timer;
	SET_TIMER_MS(timer, nTime);
	while(CHK_TIMER_MS(timer));
}

uint32_t Millis(void)
{
    return MillisCounter;
}

/*
void Increase10Micros(void)
{
    MicrosCounter += 10;
}
*/

uint32_t Micros(void)
{
    return TIM5->CNT;
}



//
// Timer 5 - 32bit - reference counter
//
void InitTimer5(int scale)
{
  TIM_TimeBaseInitTypeDef TIM_TimeBase_InitStructure;
  //NVIC_InitTypeDef NVIC_InitStructure;

  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);

  TIM_TimeBase_InitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBase_InitStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBase_InitStructure.TIM_Period = 0xFFFFFFFFl;
  TIM_TimeBase_InitStructure.TIM_Prescaler = scale;
  TIM_TimeBaseInit(TIM5, &TIM_TimeBase_InitStructure);

  /*TIM_ITConfig(TIM5, TIM_IT_Update, ENABLE);

  NVIC_InitStructure.NVIC_IRQChannel = TIM5_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0F;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0F;
  NVIC_Init(&NVIC_InitStructure);
*/
  TIM_Cmd(TIM5, ENABLE);
}
uint32_t ReadTimer5(void)
{
	return TIM5->CNT;
}



//
// Timer 4 - 16bit - 250ns pulse counter
//
void InitTimer4(int scale)
{
  TIM_TimeBaseInitTypeDef TIM_TimeBase_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);

  TIM_SelectOnePulseMode(TIM4, TIM_OPMode_Single);

  TIM_TimeBase_InitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBase_InitStructure.TIM_CounterMode = TIM_CounterMode_Down;
  TIM_TimeBase_InitStructure.TIM_Period = 0xffff;
  TIM_TimeBase_InitStructure.TIM_Prescaler = scale /4;			// 250ns
  TIM_TimeBaseInit(TIM4, &TIM_TimeBase_InitStructure);

  TIM_ITConfig(TIM4, TIM_IT_Update, ENABLE);

  NVIC_InitStructure.NVIC_IRQChannel = TIM4_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x02;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x00;
  NVIC_Init(&NVIC_InitStructure);

  TIM_Cmd(TIM4, ENABLE);
}
void TriggerTimer4(int cnt)
{
	//TIM4->CNT = 0;
	//TIM4->ARR = cnt;
	//TIM4->CR1 |= TIM_CR1_CEN;
}


//
// Timer interrupt handler
//
volatile uint32_t cntT4 = 0;
volatile uint8_t runFlg = 0;							// run flag for virtual 6502
void VcpuStep(void);
void TIM4_IRQHandler(void)
{
	//int32_t	diff;
	uint16_t delay = 1;

	//TIM_ClearITPendingBit(TIM4, TIM_IT_Update);
	TIM4->SR = (uint16_t)(~TIM_IT_Update);

	if(runFlg)
	{
		//CPU_normExecute(1);
/*
		if(sysTicker < TIM5->CNT)
			sysTicker = TIM5->CNT;

		VcpuStep();
		diff = (int32_t)(sysTicker - TIM5->CNT);
		if(diff > 3)
		{
			// slow down ...
			if(diff > 0x40)	diff = 0x40;
			delay = (diff -2) *4;
		}
*/
	}

	//SetLED(LED_BLUE, 3);
	//cntT4++;

	//TIM4->ARR = 0xFFFF;
	TIM4->CR1 |= TIM_CR1_CEN;
	TIM4->CNT = delay;
}



//
// Initialize Stimer module
//
void InitStimer(void)
{
	RCC_ClocksTypeDef RCC_Clocks;
	int scale;

	/* SysTick end of count event each 1ms */
	RCC_GetClocksFreq(&RCC_Clocks);
	SysTick_Config(RCC_Clocks.SYSCLK_Frequency / 1000);
	scale = RCC_Clocks.SYSCLK_Frequency / 2000000l;

	ResetStimer();
	InitTimer4(scale);
	InitTimer5(scale);
}
