/**********************************************************************************/
/*    Testcode for Discovery 32F4 board											  */
/*	  Board: STM32F407 Discovery				             					  */
/*    Manufacture: ST		                                                   	  */
/*	  COPYRIGHT (C) 2012														  */
/*    Designed by: Ing. Thomas Winkler			                                  */
/*    Built with CooCox CoIDE v1.5.0  					                          */
/**********************************************************************************/

#include "stm32f4xx.h"

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <strings.h>
#include <malloc.h>

#include "sdcard.h"
#include "uart.h"
#include "fswrapper.h"


//#include "ff.h"
//#include "SD_test.h"



#define	F_DIAG
#define	F_TEST_DIR



static 	FATFS  FileSystemObject;

//
// list directory of SD card
//
int SearchImageFile(char *buf, int buflen)
{
	FRESULT	rc;
	DIR		dir;
    FILINFO fno;
    char *fn;   /* This function is assuming non-Unicode cfg. */
#if _USE_LFN
    static char lfn[_MAX_LFN + 1];
    fno.lfname = lfn;
    fno.lfsize = sizeof(lfn);
#endif

	printf("--- searching for imagefile\n");

	if((rc=f_opendir(&dir, "0:")) != FR_OK)
	{
		printf("failed to execute f_opendir(). rc=%d\n", rc);
		return -1;
	}

	//i = strlen(path);
    for (;;)
    {
         rc = f_readdir(&dir, &fno);                   /* Read a directory item */
         if (rc != FR_OK || fno.fname[0] == 0) break;  /* Break on error or end of dir */
         if (fno.fname[0] == '.') continue;             /* Ignore dot entry */
#if _USE_LFN
         fn = *fno.lfname ? fno.lfname : fno.fname;
#else
         fn = fno.fname;
#endif
         if (fno.fattrib & AM_DIR)
         {
        	 // It is a directory
         } else
         {
        	 // It is a file.
        	 int 	l;
        	 char	*ext;
             l = strlen(fn);
             ext = fn + l -3;
             if(stricmp(ext, "d64") == 0)
            	 ;
             else if(stricmp(ext, "g64") == 0)
            	 ;
             else	continue;

             strncpy(buf, fn, buflen);
             return 0;
         }
    }
	return 1;
}



//
// read file into memory
//
int SDcardReadFile(char *fnam, void *buf, size_t len)
{
	FILE	*fh;
	int		rc = 0;
	DWORD	size;

	if((fh = fopen(fnam, "r")) == NULL)
		return -1;

	fseek(fh, 0, SEEK_END);
	size = ftell(fh);
	fseek(fh, 0, SEEK_SET);

	if(len == 0)
		len = size;

	if(len == size)
	{
		if(fread(buf, len, 1, fh) != 1)
			rc = -3;
	}
	else
		rc = -2;

	fclose(fh);
	return rc;
}




//
// list directory of SD card
//
int PrintDir()
{
	FRESULT	rc;
	//DWORD	clusters, tot_sect, fre_sect;
	DIR		dir;
    FILINFO fno;
//	FATFS	*fso;
    char *fn;   /* This function is assuming non-Unicode cfg. */
#if _USE_LFN
    static char lfn[_MAX_LFN + 1];
    fno.lfname = lfn;
    fno.lfsize = sizeof(lfn);
#endif

	printf("--- reading directory\n");

	if((rc=f_opendir(&dir, "0:")) != FR_OK)
	{
		printf("failed to execute f_opendir(). rc=%d\n", rc);
		return -1;
	}

	//i = strlen(path);
    for (;;) {
         rc = f_readdir(&dir, &fno);                   /* Read a directory item */
         if (rc != FR_OK || fno.fname[0] == 0) break;  /* Break on error or end of dir */
         if (fno.fname[0] == '.') continue;             /* Ignore dot entry */
#if _USE_LFN
         fn = *fno.lfname ? fno.lfname : fno.fname;
#else
         fn = fno.fname;
#endif
         if (fno.fattrib & AM_DIR) {                    /* It is a directory */
             printf("<%s>\n", fn);
         } else {                                       /* It is a file. */
             printf("%s\n", fn);
         }
     }

	return 0;
}


#ifdef F_DIAG

//
// write textfile
//
static int WriteTextFile(char *fnam)
{
	FILE	*fh;

	if((fh = fopen(fnam, "w")) == NULL)
		return -1;

	fputs("Zeile 1\n", fh);
	fputc('Z', fh);
	fputc('e', fh);
	fputs("ile 2\n", fh);
	fprintf(fh, "Zeile %d\n", 3);
	fclose(fh);
	return 0;
}


//
// type textfile
//
static int TypeTextFile(char *fnam)
{
	FILE	*fh;
	char	buffer[128];

	if((fh = fopen(fnam, "r")) == NULL)
		return -1;

	while(!feof(fh))
	{
		if(fgets(buffer, sizeof(buffer), fh) != NULL)
		{
			printf("%s", buffer);
		}
	}
	fclose(fh);
	return 0;
}


//
// test code for SD read/write
//
static int TestSD(void)
{
	printf("--- read/write test\n");
	remove("0:/test.txt");

	if(WriteTextFile("0:/test.txt"))
		printf("error writing test textfile\n");
	else if(TypeTextFile("0:/test.txt"))
		printf("error printing test textfile\n");
	else
		printf("textfile test ok.\n");

	return 0;
}
#endif




//
// init access to SD card
//
int SDcardInit()
{
	FRESULT	rc;
	DWORD	clusters, tot_sect, fre_sect;
	FATFS	*fso;

	printf("-----------------\n");
	printf("initialize SDIO system ...\n");

	if((rc=f_mount(0, &FileSystemObject)) != FR_OK)
	{
		printf("failed to execute f_mount(). rc=%d\n", rc);
		return -1;
	}
	if((rc=f_getfree("0:", &clusters, &fso)) == FR_OK)
		printf("drive 0 ok\n");
	else
	{
		printf("failed to execute f_getfree(). rc=%d\n", rc);
		return -1;
	}

    // Get total sectors and free sectors
    tot_sect = (fso->n_fatent - 2) * fso->csize;
    fre_sect = clusters * fso->csize;

    /* Print free space in unit of KB (assuming 512 bytes/sector) */
    printf("%lu KB total drive space.\n"
           "%lu KB available.\n",
           fre_sect / 2, tot_sect / 2);

    printf("SDIO system ok, free clusters: %ld\n", clusters);

#ifdef F_DIAG
	return TestSD();
#else
#endif
#ifdef F_TEST_DIR
	PrintDir();
#endif


	return 0;
}

